function [auc thresh tp fp] = get_roc(C, X1, X2, A)
    num_test = length(C);
    display(sprintf('\t\tTesting with %d pairs', num_test));
    
    pair1 = C(:, 1);
    pair2 = C(:, 2);
    pair_type = (C(:, 3) == 1); % 1 = matching, -1 = non-matching
    
    display('        Compute distances');
    dist = zeros(num_test, 1);
    for i = 1:num_test
        t = X1(i, :) - X2(i, :);
        dist(i) = t * A * t';
    end
    
    display('        Draw roc curves');
    min_dist = max(min(dist(:)) - 1, 0);
    max_dist = max(dist(:));    
    thresh = linspace(min_dist, max_dist);
    n = length(thresh);
    
    total_true_pairs = sum(pair_type(:));
    total_false_pairs = num_test - total_true_pairs;
    display(sprintf('\t\tTrue pairs: %d    False pairs: %d', total_true_pairs, total_false_pairs));
    
    tp = zeros(n, 1);
    fp = zeros(n, 1);
    auc = 0;
    
    if total_true_pairs == 0 || total_false_pairs == 0
        display('ERROR!!!!!! there is no true_pair or false_pair.');
    end
    
    for i = 1:n
        j = (dist < (thresh(i) + 0.00001));

        retrieved_true_pairs = sum(j .* pair_type);
        retrieved_pairs = sum(j);
        tp(i) = retrieved_true_pairs / total_true_pairs;
        fp(i) = (retrieved_pairs - retrieved_true_pairs) / total_false_pairs;
    end
    
    for i = 2:n
        auc = auc + tp(i) * (fp(i) - fp(i - 1));
    end    
end