function new_patch = jitter_patch(patch)
    new_patch = zeros(size(patch));
    
    [N M] = size(patch);
    
    noise_position_y = randn + 0.25;
    noise_position_x = randn + 0.25;
    noise_orientation = randn + (11 / 360.0 * 2 * pi);
    noise_scale = randn + 1.12;
    noise_scale = max(noise_scale, 0.9);
    
    for i = 1:N
        for j = 1:M
            y = noise_scale * ((j - M / 2) * sin(noise_orientation) + (i - N / 2) * cos(noise_orientation)) + (N / 2);
            x = noise_scale * ((j - M / 2) * cos(noise_orientation) - (i - N / 2) * sin(noise_orientation)) + (M / 2);
            
            y = floor(y + noise_position_y);
            x = floor(x + noise_position_x);
            
            y = max(y, 1);
            x = max(x, 1);
            y = min(y, N);
            x = min(x, M);
            
            new_patch(i, j) = patch(y, x);
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Discriminant Embedding for Local Image Descriptors, G. Hua et al.
%
% In all our experiments, we have applied a synthetic jitter
% to the patches in both the training and test sets to simulate
% errors in the interest point localisation process. Such errors 
% have been suppressed in our database since we use projections of 
% bundle adjusted 3D point positions. We apply
% Gaussian noise to the position, orientation and scale of the
% patches with standard deviations of 0.25 pixels in position,
% 11 degrees in orientation and 12% in scale respectively.