function D = jittered_sift_of_image(I)
% I is a grey-scale image
% D is 1 * 128 matrix

[n, m] = size(I);

% Not fixed orientation
% FC = [m / 2; n / 2; 6; 0]; % SIFT Frame Center [x;y;scale;orientation]
% [F, D] = vl_sift(single(I), 'frames', FC, 'orientations');

% Fixed orientation
% FC = [m / 2; n / 2; (m - 2) / 12; 0];
% [F, D] = vl_sift(single(I), 'frames', FC);


% With noise
noise_position1 = randn + 0.25;
noise_position2 = randn + 0.25;
noise_orientation = randn + (11 / 360.0 * 2 * pi);
noise_scale = randn + 0.12;

% Constraint Interval
x = min(m / 2 + noise_position1, m);
x = max(1, x);
y = min(n / 2 + noise_position2, n);
y = max(1, y);
scale = min(10, (m - 2) / 12 * noise_scale);
scale = max(1, scale);

FC = [x; y; scale; noise_orientation];
[F, D] = vl_sift(single(I), 'frames', FC);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Discriminant Embedding for Local Image Descriptors, G. Hua et al.
%
% In all our experiments, we have applied a synthetic jitter
% to the patches in both the training and test sets to simulate
% errors in the interest point localisation process. Such errors 
% have been suppressed in our database since we use projections of 
% bundle adjusted 3D point positions. We apply
% Gaussian noise to the position, orientation and scale of the
% patches with standard deviations of 0.25 pixels in position,
% 11 degrees in orientation and 12% in scale respectively.