display(' ');
display('---------------------------------------');
train_SIFT = false;
train_RAW = true;
do_SIFT = false;
do_RAW = true;

dim = 1024;

%% Read data
% Training: Halfdome & Trevi
% Testing: Notre Dame
tic
display('>>> Read data');

if dim == 4096
    data_type = '';
elseif dim == 1024
    data_type = '_1024';
elseif dim == 256
    data_type = '_256';
end

patch_halfdome = load(['data_halfdome_jitter' data_type '.mat']);
patch_halfdome = patch_halfdome.Patch;
num_halfdome = length(patch_halfdome);

patch_trevi = load(['data_trevi_jitter' data_type '.mat']);
patch_trevi = patch_trevi.Patch;
num_trevi = length(patch_trevi);

dim_sift = length(patch_halfdome(1).sift);
dim_raw = length(patch_halfdome(1).raw);

X_sift = zeros(num_halfdome + num_trevi, dim_sift);
X_raw = zeros(num_halfdome + num_trevi, dim_raw);
labels = zeros(num_halfdome + num_trevi, 1);

for i = 1:num_halfdome
    labels(i) = patch_halfdome(i).label;
    X_sift(i, :) = patch_halfdome(i).sift;    
    X_raw(i, :) = patch_halfdome(i).raw;
end

label_offset = max(labels(1:num_halfdome));
for i = 1:num_trevi
    labels(i + num_halfdome) = patch_trevi(i).label + label_offset;
    X_sift(i + num_halfdome, :) = patch_trevi(i).sift;    
    X_raw(i + num_halfdome, :) = patch_trevi(i).raw;
end

clear patch_halfdome;
clear patch_trevi;


patch_notredame = load(['data_notredame_jitter' data_type '.mat']);
patch_notredame = patch_notredame.Patch;
num_notredame = length(patch_notredame);

toc


%% Determine parameters ''U'' and ''L'' for SIFT and RAW
tic
display(' ');
display('>>> Determine U and L');
X_tmp = zeros(100, dim_sift);
r = randperm(num_halfdome);
X_tmp(1:50, :) = X_sift(r(1:50), :);
r = randperm(num_trevi);
X_tmp(51:100, :) = X_sift(r(1:50) + num_halfdome, :);

Dist = distMat(X_tmp);
Dist = sort(Dist(1:end));
l_sift = Dist(100 * 100 * 0.05);
u_sift = Dist(100 * 100 * 0.95);

X_tmp = zeros(100, dim_raw);
r = randperm(num_halfdome);
X_tmp(1:50, :) = X_raw(r(1:50), :);
r = randperm(num_trevi);
X_tmp(51:100, :) = X_raw(r(1:50) + num_halfdome, :);

Dist = distMat(X_tmp);
Dist = sort(Dist(1:end));
l_raw = Dist(100 * 100 * 0.05);
u_raw = Dist(100 * 100 * 0.95);

display(sprintf('\tRaw: L(%f) U(%f)', l_raw, u_raw));
display(sprintf('\tSIFT: L(%f) U(%f)', l_sift, u_sift));

clear Dist;
clear X_tmp;
clear r;
toc

%% Training
display(' ');
display('>>> Training metrics');

display('    Pick random samples for training');
num_matching_pairs = 10000;

labels_tmp = labels(1:num_halfdome);
C_sift = pick_pairs(labels_tmp, num_matching_pairs / 4, true, l_sift, u_sift);
C_sift = [C_sift; pick_pairs(labels_tmp, num_matching_pairs / 4, false, l_sift, u_sift)];
C_raw = pick_pairs(labels_tmp, num_matching_pairs / 4, true, l_raw, u_raw);
C_raw = [C_raw; pick_pairs(labels_tmp, num_matching_pairs / 4, false, l_raw, u_raw)];

labels_tmp = labels((num_halfdome+1):end);
C_sift = [C_sift; pick_pairs(labels_tmp, num_matching_pairs / 4, true, l_sift, u_sift)];
C_sift = [C_sift; pick_pairs(labels_tmp, num_matching_pairs / 4, false, l_sift, u_sift)];
C_raw = [C_raw; pick_pairs(labels_tmp, num_matching_pairs / 4, true, l_raw, u_raw)];
C_raw = [C_raw; pick_pairs(labels_tmp, num_matching_pairs / 4, false, l_raw, u_raw)];

num_validate_pairs = 1000;
labels_tmp = labels(1:num_halfdome);
C_sift_validate = pick_pairs(labels_tmp, num_validate_pairs / 4, true, l_sift, u_sift);
C_sift_validate = [C_sift_validate; pick_pairs(labels_tmp, num_validate_pairs / 4, false, l_sift, u_sift)];
C_raw_validate = pick_pairs(labels_tmp, num_validate_pairs / 4, true, l_raw, u_raw);
C_raw_validate = [C_raw_validate; pick_pairs(labels_tmp, num_validate_pairs / 4, false, l_raw, u_raw)];

labels_tmp = labels((num_halfdome+1):end);
C_sift_validate = [C_sift_validate; pick_pairs(labels_tmp, num_validate_pairs / 4, true, l_sift, u_sift)];
C_sift_validate = [C_sift_validate; pick_pairs(labels_tmp, num_validate_pairs / 4, false, l_sift, u_sift)];
C_raw_validate = [C_raw_validate; pick_pairs(labels_tmp, num_validate_pairs / 4, true, l_raw, u_raw)];
C_raw_validate = [C_raw_validate; pick_pairs(labels_tmp, num_validate_pairs / 4, false, l_raw, u_raw)];

X1_sift = X_sift(C_sift(:, 1), :);
X2_sift = X_sift(C_sift(:, 2), :);
X1_raw = X_raw(C_raw(:, 1), :);
X2_raw = X_raw(C_raw(:, 2), :);

X1_sift_validate = X_sift(C_sift_validate(:, 1), :);
X2_sift_validate = X_sift(C_sift_validate(:, 2), :);
X1_raw_validate = X_raw(C_raw_validate(:, 1), :);
X2_raw_validate = X_raw(C_raw_validate(:, 2), :);

A_0_sift = eye(dim_sift);
A_0_raw = eye(dim_raw);
params = struct();
params = SetDefaultParams(params);
% gammas = 10.^(-3:3);
% gammas = 0.001;
gammas = 10.^(-8:0);

if train_SIFT
    gammas = 10 ^ -4;
    tic
    display('    Training A_SIFT');
    params.max_iters = 1000000;
    accs = zeros(length(gammas), 1);
    A_sifts = cell(1, length(gammas));
    for i=1:length(gammas)
        display(sprintf('\tTuning kernel learning for SIFT: gamma = %f', gammas(i)));
        params.gamma = gammas(i); 
        A_sift = ItmlAlg_tight(C_sift, X1_sift, X2_sift, A_0_sift, params);
        A_sifts{i} = A_sift;
        [accs(i) thresh_ignore tp_ignore fp_ignore] = get_roc(C_sift_validate, X1_sift_validate, X2_sift_validate, A_sift);
        display(sprintf('\tValidation mAP = %f', accs(i)));
    end
    display([accs, gammas']);
    [value_ignore, max_idx] = max(accs);
    gamma_sift = gammas(max_idx);
    display(sprintf('\tOptimal gamma value for SIFT: %f', gamma_sift));
    params.gamma = gamma_sift;
%     A_sift = ItmlAlg_tight(C_sift, X1_sift, X2_sift, A_0_sift, params);
    A_sift = A_sifts{max_idx};
    toc
else
    A_sift = eye(dim_sift);
end


if train_RAW
    gammas = 10 ^ -6;
    tic
    display('    Training A_RAW');
    params.max_iters = 100000;
    accs = zeros(length(gammas), 1);
    A_raws = cell(1, length(gammas));
    for i=1:length(gammas)
        display(sprintf('\tTuning kernel learning for RAW: gamma = %f', gammas(i)));
        params.gamma = gammas(i); 
        A_raw = ItmlAlg_tight(C_raw, X1_raw, X2_raw, A_0_raw, params);
        A_raws{i} = A_raw;
        [accs(i) thresh_ignore tp_ignore fp_ignore] = get_roc(C_raw_validate, X1_raw_validate, X2_raw_validate, A_raw);
        display(sprintf('\tValidation mAP = %f', accs(i)));
    end
    display([accs, gammas']);
    [value_ignore, max_idx] = max(accs);
    gamma_raw = gammas(max_idx);
    display(sprintf('\tOptimal gamma value for RAW: %f', gamma_raw));
    params.gamma = gamma_raw;
%     A_raw = ItmlAlg_tight(C_raw, X1_raw, X2_raw, A_0_raw, params);
    A_raw = A_raws{max_idx};
    toc
else
    A_raw = eye(dim_raw);
end


clear labels_tmp;
clear X_sift;
clear X_raw;
clear labels;

clear accs;
clear thresh_ignore;
clear fp_ignore;
clear tp_ignore;
clear value_ignore;
clear max_idx;
clear C_raw_validate;
clear C_sift_validate;
clear C_sift;
clear C_raw;
clear X1_raw_validate;
clear X1_sift_validate;
clear X2_raw_validate;
clear X2_sift_validate;

%% Testing
display(' ');
display('>>> Testing');
X_sift = zeros(num_notredame, dim_sift);
X_raw = zeros(num_notredame, dim_raw);
labels = zeros(num_notredame, 1);

for i = 1:num_notredame
    labels(i) = patch_notredame(i).label;
    X_sift(i, :) = patch_notredame(i).sift;
    X_raw(i, :) = patch_notredame(i).raw;
end

tic
display('    pick random samples for testing');
C_sift = pick_pairs(labels, 5000, true, l_sift, u_sift);
C_sift = [C_sift; pick_pairs(labels, 5000, false, l_sift, u_sift)];
C_raw = pick_pairs(labels, 5000, true, l_raw, u_raw);
C_raw = [C_raw; pick_pairs(labels, 5000, false, l_raw, u_raw)];

X1_sift = X_sift(C_sift(:, 1), :);
X2_sift = X_sift(C_sift(:, 2), :);
X1_raw = X_raw(C_raw(:, 1), :);
X2_raw = X_raw(C_raw(:, 2), :);

clear X_sift;
clear X_raw;
clear labels;
toc

if do_SIFT
    tic
    display(' ');
    display('    compute roc curves for SIFT');
    [auc_sift thresh_sift tp_sift fp_sift] = get_roc(C_sift, X1_sift, X2_sift, A_sift);
    display(sprintf('\tAUC for SIFT: %f', auc_sift));
    
    if train_SIFT
        fID = fopen('roc_ML_SIFT.txt', 'w');
        fID2 = fopen('auc_ML_SIFT.txt', 'w');
    else
        fID = fopen('roc_L2_SIFT.txt', 'w');
        fID2 = fopen('auc_L2_SIFT.txt', 'w');
    end
    fprintf(fID, 'Threshold\tFP Rate\t TP Rate\n');
    for i = 1:length(thresh_sift)
        fprintf(fID, '%.6f\t%.6f\t%.6f\n', thresh_sift(i), fp_sift(i), tp_sift(i));
    end
    fclose(fID);
    
    fprintf(fID2, '%f', auc_sift);
    fclose(fID2);

%     Dist = zeros(5000, 1);
%     for i = 1:5000
%         t = X1_sift(i, :) - X2_sift(i, :);
%         Dist(i) =  t * A_sift * t';
%     end
%     display(sprintf('average distance of true pairs = %f', sum(Dist) / 5000));
% 
%     for i = 1:5000
%         t = X1_sift(i + 5000, :) - X2_sift(i + 5000, :);
%         Dist(i) =  t * A_sift * t';
%     end
%     display(sprintf('average distance of false pairs = %f', sum(Dist) / 5000));
    
    toc
end

if do_RAW
    tic
    display(' ');
    display('    compute roc curves for RAW');
    [auc_raw thresh_raw tp_raw fp_raw] = get_roc(C_raw, X1_raw, X2_raw, A_raw);
    display(sprintf('\tAUC for RAW: %f', auc_raw));

    if train_RAW
        fID = fopen('roc_ML_RAW.txt', 'w');
        fID2 = fopen('auc_ML_RAW.txt', 'w');
    else
        fID = fopen('roc_L2_RAW.txt', 'w');
        fID2 = fopen('auc_L2_RAW.txt', 'w');
    end
    fprintf(fID, 'Threshold\tFP Rate\t TP Rate\n');
    for i = 1:length(thresh_raw)
        fprintf(fID, '%.6f\t%.6f\t%.6f\n', thresh_raw(i), fp_raw(i), tp_raw(i));
    end
    fclose(fID);

    fprintf(fID2, '%f', auc_raw);
    fclose(fID2);

%     Dist = zeros(5000, 1);
%     for i = 1:5000
%         t = X1_raw(i, :) - X2_raw(i, :);
%         Dist(i) =  t * A_raw * t';
%     end
%     display(sprintf('average distance of true pairs = %f', sum(Dist) / 5000));
% 
%     for i = 1:5000
%         t = X1_raw(i + 5000, :) - X2_raw(i + 5000, :);
%         Dist(i) =  t * A_raw * t';
%     end
%     display(sprintf('average distance of false pairs = %f', sum(Dist) / 5000));
    
    toc
end

